# Generated by Django 3.0.2 on 2020-09-06 14:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djpaddle', '0002_checkout'),
    ]

    operations = [
        migrations.AlterField(
            model_name='checkout',
            name='id',
            field=models.CharField(max_length=64, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='checkout_id',
            field=models.CharField(max_length=64),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='id',
            field=models.CharField(max_length=64, primary_key=True, serialize=False),
        ),
    ]
