# Generated by Django 3.1.5 on 2021-01-19 04:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djpaddle', '0003_increase_length_of_paddle_ids'),
    ]

    operations = [
        migrations.AlterField(
            model_name='checkout',
            name='completed',
            field=models.BooleanField(null=True),
        ),
    ]
